%% 1) Import data: in this section data for the analysis are imported in Matlab and parameters are set
% 1.1) Define path
clear all
close all
ID='C:\Users\Username\Desktop\ToShare\example\'; % please enter the directory of your excel/matlab data file
excel=false; % choose =true if data are in excel file (=false if in matlab file)
%-----------------------------------------------------------------------------------------------        
% 1.2) Read nuclei (DA) data
if excel
    file='imaris_sheet1.xlsx'; %name excel file with DA nuclei positions 
    data = xlsread ([ID file], 1); %import position (make sure the DA nuclei positions are in sheet1)  
    spots=data(:,1:3)'; % x,y and z coordiantes of the DA nuclei
    Ig2t=squeeze(spots);
else
    file='48hpf_E03_da.mat'; %enter the name matlab file with DA nuclei
    % interpret imaris tracks
    [Positions,Time,Edges]=read_Imaris_Spots([ID file]);
    tmax=max(Time); % number of frames
    avgxyz=Positions;
    Ig2t=Positions;
    spots=Positions; % x,y and z coordiantes of the DA nuclei
end
%-----------------------------------------------------------------------------------------------
% 1.3) Read nuclei (PCV) data
if excel
    data_PCV = xlsread ([ID file], 2); %import position (make sure the PCV nuclei positions are in sheet2)  
    data_PCV=data_PCV(:,1:3)'; % x,y and z coordiantes of the PCV nuclei
else
    file_PCV='48hpf_E03_pcv.mat'; %enter the name matlab file with PCV nuclei
     % interpret imaris tracks
    [Positions_PCV,Time_PCV,Edges_PCV]=read_Imaris_Spots([ID file_PCV]);
    tmax_PCV=max(Time_PCV); % number of frames  
    avgxyz_PCV=Positions_PCV;
    data_PCV =Positions_PCV; % x,y and z coordiantes of the PCV nuclei
end
%-----------------------------------------------------------------------------------------------
% 1.4) Import the actin channel only in .tif file
ID_act='C:\Users\Username\Desktop\ToShare\example\'; %enter the path of the folder of tif file
name_t1='fliLAeGFP_flknlscherry_48hpf_E03-actin.tif' %enter the name tif file (don't forget the file extention)
[vol_ch_1,nz,nt,dx,dy,dz,stackSizeX,stackSizeY] = inport_3Dheart([ID_act,name_t1]);
%----------------------------------------------------------------------------------------------
% 1.5) Define parameters
% Distance between 2 ISV in um  
L_ISV=85
%Image properties:
dx=0.216; 
dy=0.216; 
dz=0.566; 

%% 2) Segmentation of the DA (using actin channel)
% 2.1) Define roi
vol_ch1t=vol_ch_1;
close all
I=zeros(stackSizeY,stackSizeX,3,'uint8');
it=1;
vol_ch2=vol_ch1t(:,:,:,it)*0; 
vol_ch1=vol_ch1t(:,:,:,it);
[vol_ch2,vol_ch1,xy]=Define_ROI(I,vol_ch1t,vol_ch1t,nz,[]); %In this function you will be asked to draw approximatly 
% the border of the DA
%----------------------------------------------------------------------------------------------
% 2.2)Fit DA with a cylinder 
np=[1;0;0]; 
Npl0=50; %number of cross section for semi-authmatic fitting
Npe=50; %number points of each ellipse
[centroid,Rcir,np,nplATR,outerCircATR,dstep,CylCenter,CylDirection,CylRadius,CylRadiusStart,CylDirectionStart]=fit_AVC(Ig2t,np,Npl0);
%----------------------------------------------------------------------------------------------
% 2.3) Ellipse segmentation, DA fitting and orientation in the reference
% system
ellipse = zeros(3,Npl0,Npe);
Area=zeros(1,Npl0);
perim=zeros(1,Npl0);
ecc=zeros(1,Npl0);
Ix=[];
% Semi automatic segmentation of Npl0 cross sections
for ipl=1:Npl0
    [X,Y,Z,Ix,x_E,y_E,Ip,A,per,c]=ManualSegm(vol_ch1,vol_ch2,centroid(:,ipl),np(:,ipl),Npe,Ix,dx,dy,dz,Rcir,ipl);
    if ipl==Npl0/2
        Ell_x=x_E;
        Ell_y=y_E;
        Am=A;
    end
    ellipse (1,ipl,:)= X;
    ellipse (2,ipl,:)= Y;
    ellipse (3,ipl,:)= Z;
    Area(1,ipl)=A;
    perim(1,ipl)=per;
    ecc(1,ipl)=c;
end
for ipl=Ix
    [X,Y,Z]= AutoSegm(vol_ch1,vol_ch2,centroid(:,ipl),np(:,ipl),Npe,dx,dy,dz,Rcir,Ell_x,Ell_y);
    ellipse (1,ipl,:)= X;
    ellipse (2,ipl,:)= Y;
    ellipse (3,ipl,:)= Z;
end
Ix_t1=Ix;
Area(Area==0)=[];
perim(perim==0)=[];
ecc(ecc==0)=[];
bsimATR=ellipse2np(ellipse,Npl0,np(:,1));
close all
%----------------------------------------------------------------------------------------------
% 2.4)Interpolate cross sections
Nep=150 %150 points along the ellipses in our case
Npl=450; %450 cross sections were chosen in our case
gaps=-2:2:2;
[Bmean,~,~,~,~,~]=interpolate_cross_sections(Npl,Nep,Npl,Nep,bsimATR(:,1:end,:),gaps);
close all
%----------------------------------------------------------------------------------------------
% 2.5) Define common reference system and orient the vessel and the nuclei (DA, PCV) 
[b_or,Dp,spots_or,e1,e2,e3]=DA_ref_system(data_PCV,Ig2t,spots,CylCenter,CylDirection,CylDirectionStart,CylRadiusStart,Bmean)
% Re-assign index of ellipses points so that the most dorsal point of each
% ellipse (previously point Dp) is now point 1
B_or=b_or*0; % oriented for cells migration
B_or(:,:,1:size(Bmean,3)-Dp+1)=b_or(:,:,Dp:size(Bmean,3));
B_or(:,:,size(Bmean,3)-Dp+2:size(Bmean,3))=b_or(:,:,1:Dp-1);
% plot3(squeeze(B_or(1,:,1)),squeeze(B_or(2,:,1)),squeeze(B_or(3,:,1)),'ow') %you will now see the most dorsal points along the vessel in white
b_or=B_or;
%% 3) Analysis
% 3.1)Ellipses geometry: plot area/perimeter/eccentricity of the ellipses
% of the DA
close all
folder_w=[ID '/results']; %results are saved in a folder called "results" inside the folder of the embryo
mkdir (folder_w)
for i=1:50
    xc(1,i)=norm(centroid(:,i)-centroid(:,1));
end
h=figure
plot(xc(1:length(Area)),Area,'r') 
ylabel('Area [um^2]')
xlabel('AP axis [um]')
xlim([0,max(xc)])
print(h,'-dtiff',[ID 'results/' 'Area.tif']) %Area plot saved as Area.tif

h=figure
plot(xc(1:length(Area)),perim,'r')
ylabel('Perimeter [um]')
xlabel('AP axis [um]')
xlim([0,max(xc)])
print(h,'-dtiff',[ID 'results/' 'Perimeter.tif']) %Perimeter plot saved as Area.tif

h=figure
plot(xc(1:length(Area)),ecc,'r')
ylabel('eccentricity')
xlabel('AP axis [um]')
xlim([0,max(xc)])
print(h,'-dtiff',[ID 'results/' 'Eccentricity.tif']) %Eccentricity plot saved as Area.tif

%Summary of results 
name2{1}='Area [um2]'; %Area of 50 ellipses
data2(1,1:size(Area,2))=Area;
name2{2}='Perimeter [um]'; %Perimeter of 50 ellipses
data2(2,1:size(perim,2))=perim;
name2{3}='Eccentricity_t1'; %eccentricity of 50 ellipses
data2(3,1:size(ecc,2))=ecc;
name3{1}='Mean_area' %Mean area of cross sections
data3{1}=mean(Area)
name3{2}='Mean_perimeter' %Mean perimeter of cross sections
data3{2}=mean(perim)
name3{3}='Mean_eccentricity' %Mean eccentricity of cross sections
data3{3}=mean(ecc)
name3=name3';
data3=data3';
name2=name2';
%----------------------------------------------------------------------------------------------
% 3.2) Division of the DA cross sections in 4 regions with same
% lenght: A1, A2, A3 and A4 are the angles that limit the 4 regions
a1=pi/4;
a2=-pi/4;
a3=pi-pi/4;
a4=-pi+pi/4;
IxEll_limits=[];
%Re-alignment of the cross sections along the AP axis(-> center of all the cross-sections will be in 0,0)
%to compare the dorsal ventral left right regions
for ipl=1:size(b_or,2) 
    C(1)=mean(squeeze(b_or(1,ipl,:)));
    C(2)=mean(squeeze(b_or(2,ipl,:)));
    B_new(1,ipl,:)=b_or(1,ipl,:)-C(1);
    B_new(2,ipl,:)=b_or(2,ipl,:)-C(2);
    B_new(3,ipl,:)=b_or(3,ipl,:);
end
b_save=b_or; % to keep b_or (vessel before the re-alignment)
b_or=B_new; % vessel after the re-alignment
for i=1:size(b_or,2) %for each cross section
    TH(i,1:size(b_or,3)) = squeeze(cart2sph(b_or(1,i,:),b_or(2,i,:),b_or(3,i,:)))'; %angular coordin. of the points
    clear Ventral_idx
    clear Dorsal_idx
    clear Right_idx
    clear Left_idx
    Ventral_idx =find(TH(i,:)<=(a1) & TH(i,:)>=(a2));
    Dorsal_idx  =find(TH(i,:)>=(a3) | TH(i,:)<=(a4)); % ACTUNG: TH goes from -pi to pi
    if (b_or(3,1,1)<0)
        Right_idx   =find(TH(i,:)>(a1)  & TH(i,:)<(a3));
        Left_idx  =find(TH(i,:)<(a2) & TH(i,:)>(a4));
    else
        Left_idx   =find(TH(i,:)>(a1)  & TH(i,:)<(a3));
        Right_idx  =find(TH(i,:)<(a2) & TH(i,:)>(a4));
    end
    Crf=0; % approximated crf of the cross section
    for ip=2:size(b_or,3)
        Crf=Crf+norm(b_or(:,i,ip)-b_or(:,i,ip-1));
    end
    SectL=Crf/4; % lenght of the four equal parts
    P=[SectL/2,SectL/2+SectL,SectL/2+SectL+SectL,SectL/2+SectL+SectL+SectL];
    VRegionL=0; %lenght of the ventral region using angles a1 and a2 as borders
    for ip=2:size(Ventral_idx,2)
        VRegionL(ip)=VRegionL(ip-1)+norm(b_or(:,i,Ventral_idx(ip))-b_or(:,i,Ventral_idx(ip-1)));
    end
    Ven_c=VRegionL(end)/2; %half lenght of the ventral region using angles a1 and a2 as borders
    [~,inv]=(min(abs((VRegionL(:)-Ven_c))));
    Pv=(min(Ventral_idx)+inv-1); % point at the centre of ventral sector
    if (Pv==1)
        Pv=2
    end
    V=0;
    for ip=Pv:size(b_or,3)
        V(ip-Pv+2)=V(ip-Pv+1)+norm(b_or(:,i,ip)-b_or(:,i,ip-1));
    end
    [~,inv1]=(min(abs((V(:)-P(1))))); 
    inv1=inv1+floor(Pv); % delimitation of ventral sector (point 1)
    [~,ind1]=(min(abs((V(:)-P(2)))));
    ind1=ind1+floor(Pv); 
    if(ind1>=151)
        ind1=150;
    end % delimitation of dorsal sector (point 1)
    V=0;
    count=1;
    for ip=Pv:-1:2
        count=count+1;
        V(count)=V(count-1)+norm(b_or(:,i,ip)-b_or(:,i,ip-1));
    end
    [~,inv2]=(min(abs((V(:)-P(1)))));
    inv2=floor(Pv)-inv2; % delimitation of ventral sector (point 2)
    [~,ind2]=(min(abs((V(:)-P(2)))));
    ind2=floor(Pv)-ind2; % delimitation of dorsal sector (point 2)
    if ind2==0
        ind2=1;
    end
    all_in=[ind1,ind2,inv2,inv1]; 
    IxEll_limits=[IxEll_limits;all_in];
end
IxEll_limits(IxEll_limits==0)=1 %lists of points (index) of ellipses that limit the 4 regions
IxEll_limits(IxEll_limits>=size(b_or,3))=size(b_or,3)

for i=1:size(IxEll_limits,1) 
    LimitPoint1(i,1:2)=b_or(1:2,i,IxEll_limits(i,1)); %x and y coord of the delimitation point 1 along the vessel
end
MeanLimitPoint1=mean(LimitPoint1); %mean x and y  delimitation coord point 1
for i=1:size(IxEll_limits,1)
    LimitPoint2(i,1:2)=b_or(1:2,i,IxEll_limits(i,2)); %x and y coord of the delimitation point 2 along the vessel
end
MeanLimitPoint2=mean(LimitPoint2);  %mean x and y  delimitation coord point 2
for i=1:size(IxEll_limits,1)
    LimitPoint3(i,1:2)=b_or(1:2,i,IxEll_limits(i,3)); %x and y coord of the delimitation point 3 along the vessel
end
MeanLimitPoint3=mean(LimitPoint3);  %mean x and y  delimitation coord point 3
for i=1:size(IxEll_limits,1)
    LimitPoint4(i,1:2)=b_or(1:2,i,IxEll_limits(i,4)); %x and y coord of the delimitation point 4 along the vessel
end
MeanLimitPoint4=mean(LimitPoint4); %mean x and y  delimitation coord point 4
for i=1:size(IxEll_limits,1)
    zz(i)=mean(squeeze(b_or(3,i,:))');
end
MeanLimitPoints=[MeanLimitPoint1;MeanLimitPoint2;MeanLimitPoint3;MeanLimitPoint4]; % x and y coord 4 delimitation points
CC=[mean(MeanLimitPoints(:,1)),mean(MeanLimitPoints(:,2))]
B_D=[MeanLimitPoints(:,1)-CC(1),MeanLimitPoints(:,2)-CC(2)]
TH_d = squeeze(cart2sph(B_D(:,1),B_D(:,2),[0,0,0,0]'))'; % angle of the delimitation points

[~,i1]=min(abs(TH_d-a1))
A1=TH_d(i1); % new angle corresponding to pi/4
[~,i2]=min(abs(TH_d-a2))
A2=TH_d(i2); % new angle corresponding to -pi/4
[~,i3]=min(abs(TH_d-a3))
A3=TH_d(i3); % new angle corresponding to pi-pi/4
[~,i4]=min(abs(TH_d-a4))
A4=TH_d(i4); % new angle corresponding to -pi+pi/4

%Visualization of the 4 regions on the DA in different colours 
figure
h=figure(1)
hold off
quiver3(0,0,0,e1(1),e1(2),e1(3),350)
hold on
quiver3(0,0,0,e2(1),e2(2),e2(3),50)
quiver3(0,0,0,e3(1),e3(2),e3(3),50)
ylabel(['y RL'])
xlabel(['x DV'])
axis equal
plot3(squeeze(b_or(1,:,:)),squeeze(b_or(2,:,:)),squeeze(b_or(3,:,:)),'-k')
plot3((MeanLimitPoint1(1)),(MeanLimitPoint1(2)),zz(:),'.y')
plot3((MeanLimitPoint2(1)),(MeanLimitPoint2(2)),zz(:),'.g')
plot3((MeanLimitPoint3(1)),(MeanLimitPoint3(2)),zz(:),'.b')
plot3((MeanLimitPoint4(1)),(MeanLimitPoint4(2)),zz(:),'.r')
for i=1:size(b_or,2)
    TH(i,1:size(b_or,3)) = squeeze(cart2sph(b_or(1,i,:),b_or(2,i,:),b_or(3,i,:)))';
    clear Ventral_idx
    clear Dorsal_idx
    clear Right_idx
    clear Left_idx
    Ventral_idx =find(TH(i,:)<=(A1) & TH(i,:)>=(A2));
    Dorsal_idx  =find(TH(i,:)>=(A3) | TH(i,:)<=(A4)); % ACTUNG: TH goes from -pi to pi
    if (b_or(3,1,1)<0)
        Right_idx   =find(TH(i,:)>(A1)  & TH(i,:)<(A3));
        Left_idx  =find(TH(i,:)<(A2) & TH(i,:)>(A4));
    else
        Left_idx   =find(TH(i,:)>(A1)  & TH(i,:)<(A3));
        Right_idx  =find(TH(i,:)<(A2) & TH(i,:)>(A4));
    end
    plot3(squeeze(b_or(1,i,Ventral_idx)),squeeze(b_or(2,i,Ventral_idx)),squeeze(b_or(3,i,Ventral_idx)),'.y')
    if isempty(Dorsal_idx)
    else
        plot3(squeeze(b_or(1,i,Dorsal_idx)),squeeze(b_or(2,i,Dorsal_idx)),squeeze(b_or(3,i,Dorsal_idx)),'.g')
    end
    plot3(squeeze(b_or(1,i,Left_idx)),squeeze(b_or(2,i,Left_idx)),squeeze(b_or(3,i,Left_idx)),'.r')
    if isempty( Right_idx)
    else
        plot3(squeeze(b_or(1,i,Right_idx)),squeeze(b_or(2,i,Right_idx)),squeeze(b_or(3,i,Right_idx)),'.m')
    end
end
close all
% Nuclei projection on the DA to re-align the nuclei around the center of the ref system (0,0)
% Min distance to points of b_save (DA before the re-alignment of the cross sections). 
% For better visualization the nuclei are then "moved" to b_or (DA after the re-alignment of the
% cross sections)
[spots_or]=proj_nuclei(spots_or,b_save,b_or,Npl);
Ig2t=spots_or;
%----------------------------------------------------------------------------------------------
% 3.4) DA nuclei position analysis
TrackX=spots_or(1,:);
TrackY=spots_or(2,:);
TrackZ=spots_or(3,:);
CellID=[];
Minutes=[];
[data,name,Pos,THb]=CellsPosition(TrackX,TrackY,TrackZ,spots_or,b_or,A1,A2,A3,A4);
close all

% Results normalization: number of cells in each sector normalized per L_ISV
C1=[mean(b_or(1,1,:)),mean(b_or(2,1,:)),mean(b_or(3,1,:))]; %center first cross section
C2=[mean(b_or(1,end,:)),mean(b_or(2,end,:)),mean(b_or(3,end,:))]; %center last cross section
L_DA=sqrt((C1(1)-C2(1)).^2+(C1(2)-C2(2)).^2+(C1(3)-C2(3)).^2); %lenght DA
rD=L_DA/L_ISV; %ratio between DA length and ISVs distance to rescale and compare samples
data_norm=data;
tot=sum(data_norm(6:2:12)); % tot number cell in embryo
data_norm(2:2:12)=data_norm(2:2:12)/rD; %data after normalizing 
tot_norm=tot/rD; %Total number of cells in embyo after normalization
%-------------------------------------------------------------------------------
% 3.5) Plot of DA nuclei position 
figure1 = figure;
N=50;
clear p bin ind bin1 bin2 bin3 bin4 bin5
% Division in bins
Ndiv=10; %Number to divide one region (10 -> 9 bins inside 1 region)
% The following rows take the delimitation angles of each region and divide the
% region in Ndiv -1 bins
bin1=linspace(-pi,A4,Ndiv/2); %bin1 and bin5 refer to the same region
bin2=linspace(A4,A2,Ndiv); %Example: This region is between the angle A4 and A2. This space is divided in Ndiv -1 bins
bin3=linspace(A2,A1,Ndiv);
bin4=linspace(A1,A3,Ndiv);
bin5=linspace(A3,pi,Ndiv/2);
bin=[bin1,bin2(2:end),bin3(2:end),bin4(2:end),bin5(2:end)]; %summary of all angles that limit the bins 
% The following rows search for nuclei inside the bins
% middleAngle is the string containing the angles at the middle of each bin: the number of nuclei in
% the bin will be plotted at this angle
% InsideBin contains the index of nuclei inside the bin (i.e. their angular position
% is between the limits of the bin)
for ib=1:length(bin)-1
    InsideBin{ib}=find(THb>=bin(ib)&THb<bin(ib+1));
    middleAngle(ib)=(bin(ib)+bin(ib+1))/2;
end
% N contains the number of nuclei in each bin (normalized per ISV distance)
if b_or(3,1,1)<0
    for ib=1:length(bin)-1
        N(ib)=length(InsideBin{ib})/rD 
        hold on
    end
else
    for ib=length(bin)-1:-1:1
        N(length(bin)-ib)=length(InsideBin{ib})/rD
        hold on
    end
end
% Plot: the following part is to colour the different regions (left right
% dorsal and ventral
% (The 4 regions have different sizes in terms of angular amplitude but
% their archlenght is the same)
v1=[-pi,-pi,A4,A4;0,max(N),max(N),0];
fill(v1(1,:),v1(2,:),'g')
hold on
v2=[A4,A4,A2,A2;0,max(N),max(N),0]
fill(v2(1,:),v2(2,:),'r')
v3=[A2,A2,A1,A1;0,max(N),max(N),0]
fill(v3(1,:),v3(2,:),'y')
v4=[A1,A1,A3,A3;0,max(N),max(N),0]
fill(v4(1,:),v4(2,:),'m')
v5=[A3,A3,pi,pi;0,max(N),max(N),0]
fill(v5(1,:),v5(2,:),'g')
%Plot the number N of nuclei in a bin at the angle middleAngle in the middle
%of the bin
plot1 = plot(middleAngle,N,'-ok')
plot(repmat(A1,1,450),linspace(0,450,450),'-.k')
plot(repmat(A2,1,450),linspace(0,450,450),'-.k')
plot(repmat(A3,1,450),linspace(0,450,450),'-.k')
plot(repmat(A4,1,450),linspace(0,450,450),'-.k')
xlim([-pi,pi])
ylim([0,max(N)])
xlabel('TH')
ylabel('#cells')
%Plot the % of nuclei in a bin at the angle middleAngle in the middle
%of the bin
% N will be now normalized per the total number of cells between 2 ISVs
% to give the % of nuclei in each bin 
if b_or(3,1,1)<0
    for ib=1:length(bin)-1
        N(ib)=N(ib)/tot_norm; %normalize the previous value for the tot number of cells between 2 ISVs
        hold on
    end
else
    for ib=length(bin)-1:-1:1
        N(length(bin)-ib)= N(length(bin)-ib)/tot_norm;
        hold on
    end
end
figure2 = figure;          
fill(v1(1,:),v1(2,:),'g')
hold on
fill(v2(1,:),v2(2,:),'r')
fill(v3(1,:),v3(2,:),'y')
fill(v4(1,:),v4(2,:),'m')
fill(v5(1,:),v5(2,:),'g')
plot1 = plot(middleAngle,N,'-ok')
plot(repmat(A1,1,450),linspace(0,450,450),'-.k')
plot(repmat(A2,1,450),linspace(0,450,450),'-.k')
plot(repmat(A3,1,450),linspace(0,450,450),'-.k')
plot(repmat(A4,1,450),linspace(0,450,450),'-.k')
xlim([-pi,pi])
ylim([0,max(N)])
xlabel('TH')
ylabel('%cells')

%% save the Workspace
save([ID 'results/' 'matlab'])


